@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2006-2008 Henning Norén
+ * Copyright (C) 2006-2014 Henning Norén
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,9 +30,11 @@ struct p_str {
   uint8_t len;
 };
 
+static int o_len, u_len;
+
 typedef struct p_str p_str;
 
-__attribute__ ((pure)) static inline bool
+__attribute__ ((pure)) static bool
 isWhiteSpace(const int ch) {
   return (ch == 0x20 || (ch >= 0x09 && ch <= 0x0d) || ch == 0x00);
 }
@@ -630,7 +632,7 @@ parseEncrypObject(FILE *file, EncData *e) {
 	  if(!fl) { 
 	    /* BZZZT!!  This is sooo wrong but will work for most cases.
 	       only use the first length we stumble upon */
-	    if(tmp_l > 128 && tmp_l < 40 && ((tmp_l % 8) != 0)) {
+	    if(tmp_l > 256 || tmp_l < 40 || ((tmp_l % 8) != 0)) {
 	      fprintf(stderr, "WARNING: Length = %d\n", tmp_l);
 	    }
 	    e->length = tmp_l;
@@ -639,12 +641,24 @@ parseEncrypObject(FILE *file, EncData *e) {
 	}
 	break;
       case 'O':
+	/** Check if it is OE string and ignore it then */
+	ch = getc(file);
+	if(ch == 'E') {
+	  str = parseRegularString(file);
+	  if(str) {
+	    free(str->content);
+	    free(str);
+	  }
+	  break;
+	}
+	else
+	  ungetc(ch,file);
+	/** Parse O-String */
 	str = parseRegularString(file);
 	if(!str)
 	  break;
-	if(str->len != 32)
-	  fprintf(stderr, "WARNING: O-String != 32 Bytes: %d\n", str->len);
 	e->o_string = str->content;
+	o_len = str->len;
 	free(str);
 	fo = true;
 	break;
@@ -665,12 +679,24 @@ parseEncrypObject(FILE *file, EncData *e) {
 	}
 	break;
       case 'U':
+	/** Check if it is UE string and ignore it then */
+	ch = getc(file);
+	if(ch == 'E') {
+	  str = parseRegularString(file);
+	  if(str) {
+	    free(str->content);
+	    free(str);
+	  }
+	  break;
+	}
+	else
+	  ungetc(ch,file);
+	/** Parse U-string */
 	str = parseRegularString(file);
 	if(!str)
 	  break;
-	if(str->len != 32)
-	  fprintf(stderr, "WARNING: U-String != 32 Bytes: %d\n", str->len);
 	e->u_string = str->content;
+	u_len = str->len;
 	free(str);
 	fu = true;
 	break;
@@ -696,9 +722,29 @@ parseEncrypObject(FILE *file, EncData *e) {
   if(!fv)
     e->version = 0;
 
-  if(fr && cf && aesv2) {
-    e->revision = 3;
-    e->version = 2;
+  if(fr) {
+    if (e->revision >= 5 && fu && fo) {
+      if(o_len < 48) {
+	fprintf(stderr, "WARNING: O-String < 48 Bytes: %d\n", str->len);
+	fo = false;
+      }
+      if(u_len < 48) {
+	fprintf(stderr, "WARNING: U-String < 48 Bytes: %d\n", str->len);
+	fu = false;
+      }
+    }
+    else {
+      if(fu && fo) {
+	if(o_len != 32)
+	  fprintf(stderr, "WARNING: O-String != 32 Bytes: %d\n", str->len);
+	if(u_len != 32)
+	  fprintf(stderr, "WARNING: U-String != 32 Bytes: %d\n", str->len);
+      }
+      if (cf && aesv2) {
+	e->revision = 3;
+	e->version = 2;
+      }
+    }
   }
 
   if(strcmp(e->s_handler,"Standard") != 0)
@@ -728,7 +774,6 @@ findEncryptObject(FILE *file, const int e_pos, EncData *e) {
 	  ch = parseWhiteSpace(file);
 	  if(ch == 'o' && getc(file) == 'b' && getc(file) == 'j' &&
 	     parseWhiteSpace(file) == '<' && getc(file) == '<') {
-	    ftell(file);
 	    return parseEncrypObject(file, e);
 	  }
 	}
@@ -745,7 +790,8 @@ getEncryptedInfo(FILE *file, EncData *e) {
   int e_pos = -1;
   bool ret;
 
-  if(fseek(file, 0L, SEEK_END-1024))
+  /* Start checking if the block is in the end of the file */
+  if(fseek(file, -1024, SEEK_END))
     e_pos = findTrailer(file, e);
   if(e_pos < 0) {
     rewind(file);
